#!/usr/dist/pkgs/perl,v5.003/5bin.sun4/perl

require 5.003;

BEGIN {
    ($workdir) = ($0 =~ /(.*)\/.*/);
}
use lib "$workdir/../lib";
my ($workdir) = ($0 =~ /(.*)\/.*/);
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use Socket;
use strict;

$SNMP_Session::suppress_warnings = 1;
SNMP_MIB::loadoids("$workdir/../mibs", "RFC1213", "CISCO-PRODUCTS", "CISCO-STACK", "IF", "CISCO-IMAGE", "ESSWITCH");
### MAIN ###

my($community) = "public";
my($line);

# Assumptions: we assume that the host is a cisco router.
#
# IOS version has to be 11.1 or greater

sub cisco_ios_version {

    my($sess) = @_ ;
    my $res = 0;
    my(@strings, $string, $major, $minor, $patch, $train, $res);
    my(@ipadentif);

    @strings = $sess->snmpgettable('ciscoImageString');
    foreach $string (@strings) {
	if ($string =~ /^CW_VERSION/) {
	    ($res) = ($string =~ /\$(.*)\$/ );
	}
    }

    return $res;
}

# Assumptions: we assume that the host is a cisco catalyst 1900 class switch
# (Basically any hardware based on stuff bought from GrandJunction)
#
# IOS version has to be 5.35 or greater

sub cisco_cat1900_switch_ios_version {

    my($sess) = @_ ;
    my $res = 0;

    my ($source, $EPROM, $Flash) =
	$sess->snmpget('upgradeFirmwareSource',
		       'upgradeEPROMRevision',
		       'upgradeFlashBankStatus');
    if ($source == 1) {
	$res = $EPROM;
    } elsif ($source == 2) {
	$res = $Flash;
    } else {
	$res = "Invalid source value: $source";
    }

    return $res;

}

# Assumptions: we assume that the host is a cisco catalyst 5000 class switch.
#
# IOS version has to be 2.3 or greater

sub cisco_cat5k_switch_ios_version {
    
    my($sess) = @_ ;
    my $res = 0;
    my(%modules, $module, $moduleType, $moduleVersion, $res);
    my(@ipadentif);
 
    %modules = $sess->snmpgettableka(1, 'moduleType', 'moduleSwVersion');
    foreach $module (values %modules) {
	$moduleType = $module->{'moduleType'};
	$moduleVersion = $module->{'moduleSwVersion'};
	if (($moduleType == 23) || (($moduleType >= 38) && ($moduleType <= 43))
	    || ($moduleType == 57) | ($moduleType == 91) || ($moduleType == 92)
	    || ($moduleType == 200)) {
	    $res = $moduleVersion;
	}
    }

    return $res;
}


while (<>) {
    chomp;
    my($net, $netmaskstr) = split;
    my @hostip = gethostbyname($net);
    my $hostaddr = inet_aton($hostip[0]);
    $netmaskstr = "255.255.255.0" unless defined($netmaskstr);
    my $netmask = inet_aton($netmaskstr);
    my ($invert) = inet_aton("255.255.255.254") ^ $netmask;
    my ($minip) = $hostaddr & $netmask;
    my ($maxip) = $minip | $invert;
    $minip = unpack('I', $minip);
    $maxip = unpack('I', $maxip);
    
    my $packedhost;
    for ($packedhost=$minip+1; $packedhost <= $maxip; $packedhost++) {
	my ($tmp) = pack('I', $packedhost);
	my $host = inet_ntoa($tmp);

	my $session = SNMP_Simple->open ($host, $community, 161);
	$session->{'session'}->{'retries'} = 2;
	my ($sysObjectID) = $session->snmpget('sysObjectID');
	if (defined $sysObjectID) {
                my $cisco_router_sysid = '1\.3\.6\.1\.4\.1\.9\.1';
		my $cisco_switch_sysid = '1\.3\.6\.1\.4\.1\.9\.5';
		my $gj_switch_sysid = '1\.3\.6\.1\.4\.1\.437.1.1.3';
		my ($ciscotype, $ios_version);

                if ($sysObjectID =~ /^$cisco_router_sysid/) {
		    $ios_version = cisco_ios_version ($session);
		    $ciscotype = "IOS"
                }

		if ($sysObjectID =~ /^$cisco_switch_sysid/) {
		    my($switchtype) = ($sysObjectID =~ /^$cisco_switch_sysid\.(\d+)/);
		    if (($switchtype == 7)     # cat 5000
			|| ($switchtype == 12) # cat 2900
			|| ($switchtype == 17) # cat 5500
			|| ($switchtype == 29) # cat 5002
			|| ($switchtype == 34) # cat 5505
			|| ($switchtype == 35) # cat 2926
			|| ($switchtype == 36) # cat 5509
			|| ($switchtype == 38) # cat 6006
			|| ($switchtype == 39) # cat 6009
			|| ($switchtype == 42) # cat 2948
			|| ($switchtype == 44) # cat 6509
			|| ($switchtype == 45) # cat 6506
			) { 
			$ciscotype = "5k";
			$ios_version = cisco_cat5k_switch_ios_version($session);
		    } elsif (($switchtype == 18) #Cat1900
			     || ($switchtype == 20) #Cat2820
			     || ($switchtype == 21) #EtherSwtich 1420
			     || ($switchtype == 28) #Cat1900C
			     || ($switchtype == 31) #Cat1900i
			     ){
			$ciscotype = "1900";
			$ios_version = cisco_cat1900_switch_ios_version($session);
		    }
		}
		
		if ($sysObjectID =~ /^$gj_switch_sysid/) {
		    my($switchtype) = ($sysObjectID =~ /^$gj_switch_sysid\.(\d+\.\d+)/);
		    if (($switchtype eq "2.0") # GJ 2800
			|| ($switchtype eq "3.2") # Cat 2100
			|| ($switchtype eq "3.3") # Cat 2800
			){
			$ciscotype = "2800";
			$ios_version = cisco_cat1900_switch_ios_version($session);
		    }
		}

	    print "$host $sysObjectID $ciscotype $ios_version\n";
	}
	$session->close();
    }
}

exit 0;
